=== modified file 'src/bake.vala'
--- src/bake.vala	2013-05-25 07:24:20 +0000
+++ src/bake.vala	2013-05-25 07:57:56 +0000
@@ -484,7 +484,7 @@
         if (debug_enabled)
             stderr.printf ("Loading %s\n", get_relative_path (original_dir, filename));
 
-        var f = new Recipe (filename);
+        var f = new Recipe.from_file (filename);
 
         /* Children can't be new toplevel recipes */
         if (!is_toplevel && f.project_name != null)
@@ -559,6 +559,16 @@
         return result;
     }
 
+    private static Option make_built_in_option (Recipe conf_file, string id, string description, string default)
+    {
+        conf_file.set_variable ("options.%s.description".printf (id), description);
+        conf_file.set_variable ("options.%s.default".printf (id), default);
+        var option = new Option (conf_file, id);
+        options.append (option);
+
+        return option;
+    }
+
     private static void find_objects (Recipe recipe)
     {
         foreach (var id in recipe.get_variable_children ("options"))
@@ -774,7 +784,7 @@
             var filename = Path.build_filename (toplevel_dir, "Recipe");
             try
             {
-                toplevel = new Recipe (filename);
+                toplevel = new Recipe.from_file (filename);
                 if (toplevel.project_name != null)
                     break;
             }
@@ -821,23 +831,14 @@
         Recipe conf_file = null;
         try
         {
-            conf_file = new Recipe (Path.build_filename (toplevel_dir, "Recipe.conf"), false);
+            conf_file = new Recipe.from_file (Path.build_filename (toplevel_dir, "Recipe.conf"), false);
         }
         catch (Error e)
         {
             if (e is FileError.NOENT)
             {
                 need_configure = true;
-                try
-                {
-                    FileUtils.set_contents (Path.build_filename (toplevel_dir, "Recipe.conf"), "");
-                    conf_file = new Recipe ("Recipe.conf");
-                }
-                catch (Error e)
-                {
-                    printerr ("Failed to create configuration: %s\n", e.message);
-                    return Posix.EXIT_FAILURE;
-                }
+                conf_file = new Recipe ();
             }
             else
             {
@@ -858,6 +859,21 @@
             stdout.printf ("%s\n", format_error ("[Build failed]"));
             return Posix.EXIT_FAILURE;
         }
+
+        /* Load options */
+        make_built_in_option (conf_file, "install-directory", "Directory to install files to", "/");
+        var root_directory = "/";
+        make_built_in_option (conf_file, "system-config-directory", "Directory to install system configuration", Path.build_filename (root_directory, "etc"));
+        make_built_in_option (conf_file, "system-binary-directory", "Directory to install system binaries", Path.build_filename (root_directory, "sbin"));
+        make_built_in_option (conf_file, "system-library-directory", "Directory to install system libraries", Path.build_filename (root_directory, "lib"));
+        make_built_in_option (conf_file, "resource-directory", "Directory to install system libraries", Path.build_filename (root_directory, "usr"));
+        var resource_directory = Path.build_filename (root_directory, "usr");
+        make_built_in_option (conf_file, "binary-directory", "Directory to install binaries", Path.build_filename (resource_directory, "bin"));
+        make_built_in_option (conf_file, "library-directory", "Directory to install libraries", Path.build_filename (resource_directory, "lib"));
+        var data_directory = Path.build_filename (resource_directory, "share");
+        make_built_in_option (conf_file, "data-directory", "Directory to install data", data_directory);
+        make_built_in_option (conf_file, "include-directory", "Directory to install headers", Path.build_filename (resource_directory, "include"));
+        make_built_in_option (conf_file, "project-data-directory", "Directory to install project files to", Path.build_filename (data_directory, toplevel.project_name));
         find_objects (toplevel);
 
         /* Make the configuration the toplevel file so everything inherits from it */
@@ -995,37 +1011,6 @@
             if (option.value == null && option.default != null)
                 option.value = option.default;
 
-        /* Derived values */
-        var root_directory = conf_file.get_variable ("root-directory");
-        if (root_directory == null)
-        {
-            root_directory = "/";
-            conf_file.set_variable ("root-directory", root_directory);
-        }
-        var resource_directory = conf_file.get_variable ("resource-directory");
-        if (resource_directory == null)
-        {
-            resource_directory = Path.build_filename (root_directory, "usr");
-            conf_file.set_variable ("resource-directory", resource_directory);
-        }
-        if (conf_file.get_variable ("system-config-directory") == null)
-            conf_file.set_variable ("system-config-directory", Path.build_filename (root_directory, "etc"));
-        if (conf_file.get_variable ("system-binary-directory") == null)
-            conf_file.set_variable ("system-binary-directory", Path.build_filename (root_directory, "sbin"));
-        if (conf_file.get_variable ("system-library-directory") == null)
-            conf_file.set_variable ("system-library-directory", Path.build_filename (root_directory, "lib"));
-        if (conf_file.get_variable ("binary-directory") == null)
-            conf_file.set_variable ("binary-directory", Path.build_filename (resource_directory, "bin"));
-        if (conf_file.get_variable ("library-directory") == null)
-            conf_file.set_variable ("library-directory", Path.build_filename (resource_directory, "lib"));
-        if (conf_file.get_variable ("data-directory") == null)
-            conf_file.set_variable ("data-directory", Path.build_filename (resource_directory, "share"));
-        if (conf_file.get_variable ("include-directory") == null)
-            conf_file.set_variable ("include-directory", Path.build_filename (resource_directory, "include"));
-        var data_directory = conf_file.get_variable ("data-directory");
-        if (conf_file.get_variable ("project-data-directory") == null)
-            conf_file.set_variable ("project-data-directory", Path.build_filename (data_directory, "$(project.name)"));
-
         /* Find the recipe in the current directory */
         var recipe = toplevel;
         while (recipe.dirname != original_dir)

=== modified file 'src/recipe.vala'
--- src/recipe.vala	2013-05-25 07:24:20 +0000
+++ src/recipe.vala	2013-05-25 07:57:56 +0000
@@ -36,22 +36,20 @@
     {
         owned get
         {
-            var dir = get_variable ("install-directory");
-            if (dir == null || Path.is_absolute (dir))
+            var dir = get_variable ("options.install-directory");
+            if (Path.is_absolute (dir))
                 return dir;
             return Path.build_filename (original_dir, dir);
         }
     }
 
-    public string source_directory { owned get { return get_variable ("source-directory"); } }
-    public string top_source_directory { owned get { return get_variable ("top-source-directory"); } }
-    public string binary_directory { owned get { return get_variable ("binary-directory"); } }
-    public string system_binary_directory { owned get { return get_variable ("system-binary-directory"); } }
-    public string library_directory { owned get { return get_variable ("library-directory"); } }
-    public string system_library_directory { owned get { return get_variable ("system-library-directory"); } }
-    public string data_directory { owned get { return get_variable ("data-directory"); } }
-    public string include_directory { owned get { return get_variable ("include-directory"); } }
-    public string project_data_directory { owned get { return get_variable ("project-data-directory"); } }
+    public string binary_directory { owned get { return get_variable ("options.binary-directory"); } }
+    public string system_binary_directory { owned get { return get_variable ("options.system-binary-directory"); } }
+    public string library_directory { owned get { return get_variable ("options.library-directory"); } }
+    public string system_library_directory { owned get { return get_variable ("options.system-library-directory"); } }
+    public string data_directory { owned get { return get_variable ("options.data-directory"); } }
+    public string include_directory { owned get { return get_variable ("options.include-directory"); } }
+    public string project_data_directory { owned get { return get_variable ("options.project-data-directory"); } }
 
     public string project_name { owned get { return get_variable ("project.name"); } }
     public string project_version { owned get { return get_variable ("project.version"); } }
@@ -70,12 +68,17 @@
         owned get { return toplevel.get_build_path (release_name); }
     }
 
-    public Recipe (string filename, bool allow_rules = true) throws FileError, RecipeError
+    public Recipe ()
     {
-        this.filename = filename;
-
         variable_names = new List<string> ();
         variables = new HashTable<string, string> (str_hash, str_equal);
+    }
+
+    public Recipe.from_file (string filename, bool allow_rules = true) throws FileError, RecipeError
+    {
+        this ();
+
+        this.filename = filename;
 
         string contents;
         FileUtils.get_contents (filename, out contents);
@@ -338,7 +341,7 @@
 
     public string get_install_path (string path)
     {
-        if (install_directory == null || install_directory == "")
+        if (install_directory == "/")
             return path;
         else
             return "%s%s".printf (install_directory, path);

=== modified file 'tests/Recipe'
--- tests/Recipe	2013-05-22 09:47:20 +0000
+++ tests/Recipe	2013-05-25 07:57:56 +0000
@@ -68,6 +68,9 @@
     }
 
     # Options
+    options-standard {
+      args = options-standard
+    }
     options-simple {
       args = options-simple
     }
@@ -549,6 +552,8 @@
           minimum-version-older/Recipe \
           minimum-version-newer/expected \
           minimum-version-newer/Recipe \
+          options-standard/expected \
+          options-standard/Recipe \
           options-simple/expected \
           options-simple/Recipe \
           options-unset/expected \

=== modified file 'tests/c++-library-install-directory/Recipe'
--- tests/c++-library-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/c++-library-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,6 +4,6 @@
 
 libraries.test {
   sources = test.cpp
-  install-directory = $(library-directory)/custom-directory
+  install-directory = $(options.library-directory)/custom-directory
 }
 

=== modified file 'tests/c++-program-install-directory/Recipe'
--- tests/c++-program-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/c++-program-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,5 +4,5 @@
 
 programs.test {
   sources = test.cpp
-  install-directory = $(binary-directory)/test
+  install-directory = $(options.binary-directory)/test
 }

=== modified file 'tests/c-library-install-directory/Recipe'
--- tests/c-library-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/c-library-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,5 +4,5 @@
 
 libraries.test {
   sources = test.c
-  install-directory = $(library-directory)/custom-directory
+  install-directory = $(options.library-directory)/custom-directory
 }

=== modified file 'tests/c-program-install-directory/Recipe'
--- tests/c-program-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/c-program-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,5 +4,5 @@
 
 programs.test {
   sources = test.c
-  install-directory = $(binary-directory)/test
+  install-directory = $(options.binary-directory)/test
 }

=== modified file 'tests/csharp-library-install-directory/Recipe'
--- tests/csharp-library-install-directory/Recipe	2013-05-05 02:52:32 +0000
+++ tests/csharp-library-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,6 +4,6 @@
 
 libraries.Test {
   sources = Test.cs
-  install-directory = $(library-directory)/custom-directory
+  install-directory = $(options.library-directory)/custom-directory
 }
 

=== modified file 'tests/csharp-program-install-directory/Recipe'
--- tests/csharp-program-install-directory/Recipe	2013-05-05 02:52:32 +0000
+++ tests/csharp-program-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,5 +4,5 @@
 
 programs.Test {
   sources = Test.cs
-  install-directory = $(binary-directory)/test
+  install-directory = $(options.binary-directory)/test
 }

=== modified file 'tests/data-complex/Recipe'
--- tests/data-complex/Recipe	2013-04-26 11:10:35 +0000
+++ tests/data-complex/Recipe	2013-05-25 07:57:56 +0000
@@ -4,5 +4,5 @@
 
 data.test {
   files = test1 test2
-  install-directory = $(project-data-directory)/test
+  install-directory = $(options.project-data-directory)/test
 }

=== modified file 'tests/data-gtk-complex/Recipe'
--- tests/data-gtk-complex/Recipe	2013-04-26 11:10:35 +0000
+++ tests/data-gtk-complex/Recipe	2013-05-25 07:57:56 +0000
@@ -5,5 +5,5 @@
 data.test {
   gtk-ui-files = test.ui
   gettext-domain = test
-  install-directory = $(project-data-directory)/ui
+  install-directory = $(options.project-data-directory)/ui
 }

=== modified file 'tests/go-program-install-directory/Recipe'
--- tests/go-program-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/go-program-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,5 +4,5 @@
 
 programs.test {
   sources = test.go
-  install-directory = $(binary-directory)/test
+  install-directory = $(options.binary-directory)/test
 }

=== modified file 'tests/haskell-program-install-directory/Recipe'
--- tests/haskell-program-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/haskell-program-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,6 +4,6 @@
 
 programs.test {
   sources = test.hs
-  install-directory = $(binary-directory)/test
+  install-directory = $(options.binary-directory)/test
 }
 

=== modified file 'tests/java-library-install-directory/Recipe'
--- tests/java-library-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/java-library-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,6 +4,6 @@
 
 libraries.test {
   sources = Test.java
-  install-directory = $(library-directory)/test
+  install-directory = $(options.library-directory)/test
 }
 

=== modified file 'tests/java-program-install-directory/Recipe'
--- tests/java-program-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/java-program-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,6 +4,6 @@
 
 programs.test {
   sources = Test.java
-  install-directory = $(binary-directory)/test
+  install-directory = $(options.binary-directory)/test
 }
 

=== added directory 'tests/options-standard'
=== added file 'tests/options-standard/Recipe'
--- tests/options-standard/Recipe	1970-01-01 00:00:00 +0000
+++ tests/options-standard/Recipe	2013-05-25 07:57:56 +0000
@@ -0,0 +1,15 @@
+project {
+  name = test-project
+}
+
+%build:
+  mkdir -p $(options.install-directory)
+  mkdir -p $(options.system-config-directory)
+  mkdir -p $(options.system-binary-directory)
+  mkdir -p $(options.system-library-directory)
+  mkdir -p $(options.resource-directory)
+  mkdir -p $(options.binary-directory)
+  mkdir -p $(options.library-directory)
+  mkdir -p $(options.data-directory)
+  mkdir -p $(options.include-directory)
+  mkdir -p $(options.project-data-directory)

=== added file 'tests/options-standard/expected'
--- tests/options-standard/expected	1970-01-01 00:00:00 +0000
+++ tests/options-standard/expected	2013-05-25 07:57:56 +0000
@@ -0,0 +1,12 @@
+!bake
+mkdir -p /
+mkdir -p /etc
+mkdir -p /sbin
+mkdir -p /lib
+mkdir -p /usr
+mkdir -p /usr/bin
+mkdir -p /usr/lib
+mkdir -p /usr/share
+mkdir -p /usr/include
+mkdir -p /usr/share/test-project
+(exit SUCCESS)

=== modified file 'tests/python-library-install-directory/Recipe'
--- tests/python-library-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/python-library-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,6 +4,6 @@
 
 libraries.test {
   sources = test.py
-  install-directory = $(library-directory)/custom-directory
+  install-directory = $(options.library-directory)/custom-directory
 }
 

=== modified file 'tests/python-program-install-directory/Recipe'
--- tests/python-program-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/python-program-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,5 +4,5 @@
 
 programs.test {
   sources = test.py
-  install-directory = $(binary-directory)/test
+  install-directory = $(options.binary-directory)/test
 }

=== modified file 'tests/vala-library-install-directory/Recipe'
--- tests/vala-library-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/vala-library-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,5 +4,5 @@
 
 libraries.test {
   sources = test.vala
-  install-directory = $(library-directory)/custom-directory
+  install-directory = $(options.library-directory)/custom-directory
 }

=== modified file 'tests/vala-program-install-directory/Recipe'
--- tests/vala-program-install-directory/Recipe	2013-04-26 11:10:35 +0000
+++ tests/vala-program-install-directory/Recipe	2013-05-25 07:57:56 +0000
@@ -4,5 +4,5 @@
 
 programs.test {
   sources = test.vala
-  install-directory = $(binary-directory)/test
+  install-directory = $(options.binary-directory)/test
 }

